# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .diagnosis_type_code import DiagnosisTypeCode

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class DiagnosisCreate(pydantic.BaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None, description="Empty string not allowed.")
    code_type: DiagnosisTypeCode = pydantic.Field(
        description="Typically, providers submitting claims to Candid are using ICD-10 diagnosis codes. If you are using ICD-10 codes, the primary diagnosis code listed on the claim should use the ABK code_type. If more than one diagnosis is being submitted on a claim, please use ABF for the rest of the listed diagnoses. If you are using ICD-9 diagnosis codes, use BK and BF for the principal and following diagnosis code(s) respectively."
    )
    code: str = pydantic.Field(
        description=(
            "Empty string not allowed.\n"
            "Should be of the appropriate format for the provided `code_type`.\n"
            "Must obey the ICD-10 format if an ICD-10 code_type is provided, specifically:\n"
            "\n"
            "- Letter\n"
            "- Digit\n"
            "- Digit or the letter `A` or `B`\n"
            "- (Optional) Period `.`\n"
            "- Up to 4 (or as few as 0) letters and digits\n"
        )
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import (
    OrganizationServiceFacility,
    OrganizationServiceFacilityCreate,
    OrganizationServiceFacilityId,
    OrganizationServiceFacilityPage,
    OrganizationServiceFacilityUpdate,
    ServiceFacilityMode,
    ServiceFacilityOperationalStatus,
    ServiceFacilityPhysicalType,
    ServiceFacilityStatus,
    ServiceFacilityType,
)

__all__ = [
    "OrganizationServiceFacility",
    "OrganizationServiceFacilityCreate",
    "OrganizationServiceFacilityId",
    "OrganizationServiceFacilityPage",
    "OrganizationServiceFacilityUpdate",
    "ServiceFacilityMode",
    "ServiceFacilityOperationalStatus",
    "ServiceFacilityPhysicalType",
    "ServiceFacilityStatus",
    "ServiceFacilityType",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .....commons.types.date import Date

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class DateUpdate_Set(pydantic.BaseModel):
    type: typing_extensions.Literal["set"]
    value: Date

    class Config:
        frozen = True
        smart_union = True


class DateUpdate_Remove(pydantic.BaseModel):
    type: typing_extensions.Literal["remove"]

    class Config:
        frozen = True
        smart_union = True


DateUpdate = typing.Union[DateUpdate_Set, DateUpdate_Remove]



# This file was auto-generated by Fern from our API Definition.

from .claim_adjustment_group_codes import ClaimAdjustmentGroupCodes
from .claim_id import ClaimId
from .claim_submission_payer_responsibility_type import ClaimSubmissionPayerResponsibilityType
from .date import Date
from .date_range_optional_end import DateRangeOptionalEnd
from .decimal import Decimal
from .delay_reason_code import DelayReasonCode
from .email import Email
from .emr_payer_crosswalk import EmrPayerCrosswalk
from .encounter_external_id import EncounterExternalId
from .encounter_id import EncounterId
from .entity_not_found_error_message import EntityNotFoundErrorMessage
from .error_message import ErrorMessage
from .facility_type_code import FacilityTypeCode
from .http_service_unavailable_error_message import HttpServiceUnavailableErrorMessage
from .insurance_type_code import InsuranceTypeCode
from .invoice_id import InvoiceId
from .link_url import LinkUrl
from .npi import Npi
from .organization_id import OrganizationId
from .organization_not_authorized_error_message import OrganizationNotAuthorizedErrorMessage
from .page_token import PageToken
from .patient_external_id import PatientExternalId
from .patient_relationship_to_insured_code_all import PatientRelationshipToInsuredCodeAll
from .phone_number import PhoneNumber
from .phone_number_type import PhoneNumberType
from .procedure_modifier import ProcedureModifier
from .provider_id import ProviderId
from .region_national import RegionNational
from .region_states import RegionStates
from .regions import Regions, Regions_National, Regions_States
from .removable_date_range_optional_end import (
    RemovableDateRangeOptionalEnd,
    RemovableDateRangeOptionalEnd_DateRange,
    RemovableDateRangeOptionalEnd_Remove,
)
from .request_validation_error import RequestValidationError
from .resource_page import ResourcePage
from .service_line_id import ServiceLineId
from .service_line_units import ServiceLineUnits
from .sort_direction import SortDirection
from .source_of_payment_code import SourceOfPaymentCode
from .state import State
from .street_address_base import StreetAddressBase
from .street_address_long_zip import StreetAddressLongZip
from .street_address_short_zip import StreetAddressShortZip
from .task_assignment_id import TaskAssignmentId
from .task_id import TaskId
from .task_note_id import TaskNoteId
from .tax_id import TaxId
from .unauthorized_error_message import UnauthorizedErrorMessage
from .unprocessable_entity_error_message import UnprocessableEntityErrorMessage
from .updates_disabled_due_to_external_system_integration_error_message import (
    UpdatesDisabledDueToExternalSystemIntegrationErrorMessage,
)
from .user_id import UserId
from .work_queue_id import WorkQueueId

__all__ = [
    "ClaimAdjustmentGroupCodes",
    "ClaimId",
    "ClaimSubmissionPayerResponsibilityType",
    "Date",
    "DateRangeOptionalEnd",
    "Decimal",
    "DelayReasonCode",
    "Email",
    "EmrPayerCrosswalk",
    "EncounterExternalId",
    "EncounterId",
    "EntityNotFoundErrorMessage",
    "ErrorMessage",
    "FacilityTypeCode",
    "HttpServiceUnavailableErrorMessage",
    "InsuranceTypeCode",
    "InvoiceId",
    "LinkUrl",
    "Npi",
    "OrganizationId",
    "OrganizationNotAuthorizedErrorMessage",
    "PageToken",
    "PatientExternalId",
    "PatientRelationshipToInsuredCodeAll",
    "PhoneNumber",
    "PhoneNumberType",
    "ProcedureModifier",
    "ProviderId",
    "RegionNational",
    "RegionStates",
    "Regions",
    "Regions_National",
    "Regions_States",
    "RemovableDateRangeOptionalEnd",
    "RemovableDateRangeOptionalEnd_DateRange",
    "RemovableDateRangeOptionalEnd_Remove",
    "RequestValidationError",
    "ResourcePage",
    "ServiceLineId",
    "ServiceLineUnits",
    "SortDirection",
    "SourceOfPaymentCode",
    "State",
    "StreetAddressBase",
    "StreetAddressLongZip",
    "StreetAddressShortZip",
    "TaskAssignmentId",
    "TaskId",
    "TaskNoteId",
    "TaxId",
    "UnauthorizedErrorMessage",
    "UnprocessableEntityErrorMessage",
    "UpdatesDisabledDueToExternalSystemIntegrationErrorMessage",
    "UserId",
    "WorkQueueId",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.encounter_id import EncounterId
from .....commons.types.task_id import TaskId
from ...commons.types.task_category import TaskCategory
from ...commons.types.task_status import TaskStatus
from ...commons.types.task_type import TaskType
from .task_assignment import TaskAssignment
from .task_note import TaskNote

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Task(pydantic.BaseModel):
    task_id: TaskId
    encounter_id: EncounterId
    task_type: TaskType
    description: str
    blocks_claim_submission: bool
    external_id: str
    patient_name: str
    patient_external_id: str
    payer_name: typing.Optional[str] = None
    payer_id: typing.Optional[str] = None
    status: TaskStatus
    notes: typing.List[TaskNote]
    created_at: dt.datetime
    updated_at: dt.datetime = pydantic.Field(description="The time of most recent update to the task only")
    agg_updated_at: dt.datetime = pydantic.Field(
        description="The time of most recent update to the task or any of its notes"
    )
    date_of_service: dt.date
    assignments: typing.List[TaskAssignment]
    category: typing.Optional[TaskCategory] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .create_write_offs_response import CreateWriteOffsResponse
from .insurance_write_off import InsuranceWriteOff
from .insurance_write_off_create import InsuranceWriteOffCreate
from .insurance_write_off_reason import InsuranceWriteOffReason
from .insurance_write_off_target import (
    InsuranceWriteOffTarget,
    InsuranceWriteOffTarget_BillingProviderId,
    InsuranceWriteOffTarget_ClaimId,
    InsuranceWriteOffTarget_ServiceLineId,
)
from .patient_write_off import PatientWriteOff
from .patient_write_off_create import PatientWriteOffCreate
from .patient_write_off_reason import PatientWriteOffReason
from .write_off import WriteOff, WriteOff_Insurance, WriteOff_Patient
from .write_off_create import WriteOffCreate, WriteOffCreate_Insurance, WriteOffCreate_Patient
from .write_off_id import WriteOffId
from .write_off_sort_field import WriteOffSortField
from .write_offs_page import WriteOffsPage

__all__ = [
    "CreateWriteOffsResponse",
    "InsuranceWriteOff",
    "InsuranceWriteOffCreate",
    "InsuranceWriteOffReason",
    "InsuranceWriteOffTarget",
    "InsuranceWriteOffTarget_BillingProviderId",
    "InsuranceWriteOffTarget_ClaimId",
    "InsuranceWriteOffTarget_ServiceLineId",
    "PatientWriteOff",
    "PatientWriteOffCreate",
    "PatientWriteOffReason",
    "WriteOff",
    "WriteOffCreate",
    "WriteOffCreate_Insurance",
    "WriteOffCreate_Patient",
    "WriteOffId",
    "WriteOffSortField",
    "WriteOff_Insurance",
    "WriteOff_Patient",
    "WriteOffsPage",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class RefundReason(str, enum.Enum):
    OVERCHARGED = "OVERCHARGED"

    def visit(self, overcharged: typing.Callable[[], T_Result]) -> T_Result:
        if self is RefundReason.OVERCHARGED:
            return overcharged()



# This file was auto-generated by Fern from our API Definition.

from .types import (
    BillingProvider,
    EncounterProvider,
    EncounterProviderBase,
    ProviderId,
    ReferringProvider,
    RenderingProvider,
)

__all__ = [
    "BillingProvider",
    "EncounterProvider",
    "EncounterProviderBase",
    "ProviderId",
    "ReferringProvider",
    "RenderingProvider",
]



# This file was auto-generated by Fern from our API Definition.

from ....core.api_error import ApiError
from ..types.request_validation_error import RequestValidationError


class HttpRequestValidationError(ApiError):
    def __init__(self, body: RequestValidationError):
        super().__init__(status_code=422, body=body)



# This file was auto-generated by Fern from our API Definition.

from .types import Era, EraBase, EraId, EraNotFullyProcessedErrorMessage
from .errors import EraNotFullyProcessedError

__all__ = ["Era", "EraBase", "EraId", "EraNotFullyProcessedError", "EraNotFullyProcessedErrorMessage"]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class IdentifierCode(str, enum.Enum):
    MCR = "MCR"
    MCD = "MCD"

    def visit(self, mcr: typing.Callable[[], T_Result], mcd: typing.Callable[[], T_Result]) -> T_Result:
        if self is IdentifierCode.MCR:
            return mcr()
        if self is IdentifierCode.MCD:
            return mcd()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....organization_providers.resources.v_2.types.organization_provider import OrganizationProvider
from .....payers.resources.v_3.types.payer import Payer
from .contract_base import ContractBase
from .contract_id import ContractId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Contract(ContractBase):
    contract_id: ContractId
    contracting_provider: OrganizationProvider = pydantic.Field(description="The provider under contract")
    provider_count: int = pydantic.Field(
        description="The number of linked providers who can render medical services under this contract"
    )
    payer: typing.Optional[Payer] = pydantic.Field(default=None, description="The insurance company under contract")

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from ....commons.types.email import Email
from ....commons.types.encounter_id import EncounterId
from ....commons.types.phone_number import PhoneNumber
from ....commons.types.street_address_short_zip import StreetAddressShortZip
from .errors.encounter_has_existing_guarantor_error import EncounterHasExistingGuarantorError
from .types.encounter_has_existing_guarantor_error_type import EncounterHasExistingGuarantorErrorType
from .types.guarantor import Guarantor
from .types.guarantor_create import GuarantorCreate
from .types.guarantor_id import GuarantorId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(self, encounter_id: EncounterId, *, request: GuarantorCreate) -> Guarantor:
        """
        Creates a new guarantor and returns the newly created Guarantor object.

        Parameters:
            - encounter_id: EncounterId.

            - request: GuarantorCreate.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/guarantors/v1/{encounter_id}"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Guarantor, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterHasExistingGuarantorError":
                raise EncounterHasExistingGuarantorError(
                    pydantic.parse_obj_as(EncounterHasExistingGuarantorErrorType, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, guarantor_id: GuarantorId) -> Guarantor:
        """
        Retrieves a guarantor by its `guarantor_id`.

        Parameters:
            - guarantor_id: GuarantorId.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/guarantors/v1/{guarantor_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Guarantor, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        guarantor_id: GuarantorId,
        *,
        first_name: typing.Optional[str] = OMIT,
        last_name: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        date_of_birth: typing.Optional[dt.date] = OMIT,
        address: typing.Optional[StreetAddressShortZip] = OMIT,
        phone_numbers: typing.Optional[typing.List[PhoneNumber]] = OMIT,
        phone_consent: typing.Optional[bool] = OMIT,
        email: typing.Optional[Email] = OMIT,
        email_consent: typing.Optional[bool] = OMIT,
    ) -> Guarantor:
        """
        Updates a guarantor by its `guarantor_id`.

        Parameters:
            - guarantor_id: GuarantorId.

            - first_name: typing.Optional[str].

            - last_name: typing.Optional[str].

            - external_id: typing.Optional[str]. A unique identifier for the guarantor assigned by an external system.

            - date_of_birth: typing.Optional[dt.date]. Date formatted as YYYY-MM-DD; eg: 2019-08-25.

            - address: typing.Optional[StreetAddressShortZip].

            - phone_numbers: typing.Optional[typing.List[PhoneNumber]].

            - phone_consent: typing.Optional[bool].

            - email: typing.Optional[Email].

            - email_consent: typing.Optional[bool].
        """
        _request: typing.Dict[str, typing.Any] = {}
        if first_name is not OMIT:
            _request["first_name"] = first_name
        if last_name is not OMIT:
            _request["last_name"] = last_name
        if external_id is not OMIT:
            _request["external_id"] = external_id
        if date_of_birth is not OMIT:
            _request["date_of_birth"] = date_of_birth
        if address is not OMIT:
            _request["address"] = address
        if phone_numbers is not OMIT:
            _request["phone_numbers"] = phone_numbers
        if phone_consent is not OMIT:
            _request["phone_consent"] = phone_consent
        if email is not OMIT:
            _request["email"] = email
        if email_consent is not OMIT:
            _request["email_consent"] = email_consent
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/guarantors/v1/{guarantor_id}"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Guarantor, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(self, encounter_id: EncounterId, *, request: GuarantorCreate) -> Guarantor:
        """
        Creates a new guarantor and returns the newly created Guarantor object.

        Parameters:
            - encounter_id: EncounterId.

            - request: GuarantorCreate.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/guarantors/v1/{encounter_id}"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Guarantor, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterHasExistingGuarantorError":
                raise EncounterHasExistingGuarantorError(
                    pydantic.parse_obj_as(EncounterHasExistingGuarantorErrorType, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, guarantor_id: GuarantorId) -> Guarantor:
        """
        Retrieves a guarantor by its `guarantor_id`.

        Parameters:
            - guarantor_id: GuarantorId.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/guarantors/v1/{guarantor_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Guarantor, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        guarantor_id: GuarantorId,
        *,
        first_name: typing.Optional[str] = OMIT,
        last_name: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        date_of_birth: typing.Optional[dt.date] = OMIT,
        address: typing.Optional[StreetAddressShortZip] = OMIT,
        phone_numbers: typing.Optional[typing.List[PhoneNumber]] = OMIT,
        phone_consent: typing.Optional[bool] = OMIT,
        email: typing.Optional[Email] = OMIT,
        email_consent: typing.Optional[bool] = OMIT,
    ) -> Guarantor:
        """
        Updates a guarantor by its `guarantor_id`.

        Parameters:
            - guarantor_id: GuarantorId.

            - first_name: typing.Optional[str].

            - last_name: typing.Optional[str].

            - external_id: typing.Optional[str]. A unique identifier for the guarantor assigned by an external system.

            - date_of_birth: typing.Optional[dt.date]. Date formatted as YYYY-MM-DD; eg: 2019-08-25.

            - address: typing.Optional[StreetAddressShortZip].

            - phone_numbers: typing.Optional[typing.List[PhoneNumber]].

            - phone_consent: typing.Optional[bool].

            - email: typing.Optional[Email].

            - email_consent: typing.Optional[bool].
        """
        _request: typing.Dict[str, typing.Any] = {}
        if first_name is not OMIT:
            _request["first_name"] = first_name
        if last_name is not OMIT:
            _request["last_name"] = last_name
        if external_id is not OMIT:
            _request["external_id"] = external_id
        if date_of_birth is not OMIT:
            _request["date_of_birth"] = date_of_birth
        if address is not OMIT:
            _request["address"] = address
        if phone_numbers is not OMIT:
            _request["phone_numbers"] = phone_numbers
        if phone_consent is not OMIT:
            _request["phone_consent"] = phone_consent
        if email is not OMIT:
            _request["email"] = email
        if email_consent is not OMIT:
            _request["email_consent"] = email_consent
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/guarantors/v1/{guarantor_id}"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Guarantor, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.insurance_type_code import InsuranceTypeCode
from .....commons.types.source_of_payment_code import SourceOfPaymentCode

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class InsuranceCardBase(pydantic.BaseModel):
    group_number: typing.Optional[str] = pydantic.Field(default=None, description="Box 11 on the CMS-1500 claim form.")
    plan_name: typing.Optional[str] = pydantic.Field(default=None, description="Box 11c on the CMS-1500 claim form.")
    plan_type: typing.Optional[SourceOfPaymentCode] = None
    insurance_type: typing.Optional[InsuranceTypeCode] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .era import Era
from .era_base import EraBase
from .era_id import EraId
from .era_not_fully_processed_error_message import EraNotFullyProcessedErrorMessage

__all__ = ["Era", "EraBase", "EraId", "EraNotFullyProcessedErrorMessage"]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from ....commons.types.encounter_id import EncounterId
from .types.billing_note import BillingNote

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V2Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(self, *, encounter_id: EncounterId, text: str) -> BillingNote:
        """
        Parameters:
            - encounter_id: EncounterId.

            - text: str. Empty string not allowed.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/billing_notes/v2"),
            json=jsonable_encoder({"encounter_id": encounter_id, "text": text}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(BillingNote, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV2Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(self, *, encounter_id: EncounterId, text: str) -> BillingNote:
        """
        Parameters:
            - encounter_id: EncounterId.

            - text: str. Empty string not allowed.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/billing_notes/v2"),
            json=jsonable_encoder({"encounter_id": encounter_id, "text": text}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(BillingNote, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .date_range_optional_end import DateRangeOptionalEnd

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class RemovableDateRangeOptionalEnd_DateRange(DateRangeOptionalEnd):
    type: typing_extensions.Literal["date_range"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class RemovableDateRangeOptionalEnd_Remove(pydantic.BaseModel):
    type: typing_extensions.Literal["remove"]

    class Config:
        frozen = True
        smart_union = True


RemovableDateRangeOptionalEnd = typing.Union[
    RemovableDateRangeOptionalEnd_DateRange, RemovableDateRangeOptionalEnd_Remove
]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from ....commons.types.date import Date
from ....commons.types.insurance_type_code import InsuranceTypeCode
from ....commons.types.state import State
from .types.expected_network_status_response import ExpectedNetworkStatusResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def compute(
        self,
        *,
        external_patient_id: typing.Optional[str] = OMIT,
        subscriber_payer_id: str,
        subscriber_payer_name: str,
        subscriber_insurance_type: typing.Optional[InsuranceTypeCode] = OMIT,
        subscriber_plan_name: typing.Optional[str] = OMIT,
        billing_provider_npi: str,
        billing_provider_tin: str,
        rendering_provider_npi: str,
        contracted_state: State,
        date_of_service: Date,
    ) -> ExpectedNetworkStatusResponse:
        """
        Computes the expected network status given the provided information.

        Parameters:
            - external_patient_id: typing.Optional[str].

            - subscriber_payer_id: str.

            - subscriber_payer_name: str.

            - subscriber_insurance_type: typing.Optional[InsuranceTypeCode].

            - subscriber_plan_name: typing.Optional[str]. The descriptive name of the insurance plan selected by the subscriber, often indicating coverage specifics or tier.

            - billing_provider_npi: str. The National Provider Identifier (NPI) of the healthcare provider responsible for billing. A unique 10-digit identification number.

            - billing_provider_tin: str. Follow the 9-digit format of the Taxpayer Identification Number (TIN).

            - rendering_provider_npi: str. The National Provider Identifier (NPI) of the healthcare provider who delivered the services. A unique 10-digit identification number.

            - contracted_state: State. The state in which the healthcare provider has a contractual agreement with the insurance payer.

            - date_of_service: Date. Date formatted as YYYY-MM-DD; eg: 2019-08-25.

        """
        _request: typing.Dict[str, typing.Any] = {
            "subscriber_payer_id": subscriber_payer_id,
            "subscriber_payer_name": subscriber_payer_name,
            "billing_provider_npi": billing_provider_npi,
            "billing_provider_tin": billing_provider_tin,
            "rendering_provider_npi": rendering_provider_npi,
            "contracted_state": contracted_state,
            "date_of_service": date_of_service,
        }
        if external_patient_id is not OMIT:
            _request["external_patient_id"] = external_patient_id
        if subscriber_insurance_type is not OMIT:
            _request["subscriber_insurance_type"] = subscriber_insurance_type
        if subscriber_plan_name is not OMIT:
            _request["subscriber_plan_name"] = subscriber_plan_name
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/expected-network-status/v1"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ExpectedNetworkStatusResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def compute(
        self,
        *,
        external_patient_id: typing.Optional[str] = OMIT,
        subscriber_payer_id: str,
        subscriber_payer_name: str,
        subscriber_insurance_type: typing.Optional[InsuranceTypeCode] = OMIT,
        subscriber_plan_name: typing.Optional[str] = OMIT,
        billing_provider_npi: str,
        billing_provider_tin: str,
        rendering_provider_npi: str,
        contracted_state: State,
        date_of_service: Date,
    ) -> ExpectedNetworkStatusResponse:
        """
        Computes the expected network status given the provided information.

        Parameters:
            - external_patient_id: typing.Optional[str].

            - subscriber_payer_id: str.

            - subscriber_payer_name: str.

            - subscriber_insurance_type: typing.Optional[InsuranceTypeCode].

            - subscriber_plan_name: typing.Optional[str]. The descriptive name of the insurance plan selected by the subscriber, often indicating coverage specifics or tier.

            - billing_provider_npi: str. The National Provider Identifier (NPI) of the healthcare provider responsible for billing. A unique 10-digit identification number.

            - billing_provider_tin: str. Follow the 9-digit format of the Taxpayer Identification Number (TIN).

            - rendering_provider_npi: str. The National Provider Identifier (NPI) of the healthcare provider who delivered the services. A unique 10-digit identification number.

            - contracted_state: State. The state in which the healthcare provider has a contractual agreement with the insurance payer.

            - date_of_service: Date. Date formatted as YYYY-MM-DD; eg: 2019-08-25.

        """
        _request: typing.Dict[str, typing.Any] = {
            "subscriber_payer_id": subscriber_payer_id,
            "subscriber_payer_name": subscriber_payer_name,
            "billing_provider_npi": billing_provider_npi,
            "billing_provider_tin": billing_provider_tin,
            "rendering_provider_npi": rendering_provider_npi,
            "contracted_state": contracted_state,
            "date_of_service": date_of_service,
        }
        if external_patient_id is not OMIT:
            _request["external_patient_id"] = external_patient_id
        if subscriber_insurance_type is not OMIT:
            _request["subscriber_insurance_type"] = subscriber_insurance_type
        if subscriber_plan_name is not OMIT:
            _request["subscriber_plan_name"] = subscriber_plan_name
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/expected-network-status/v1"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ExpectedNetworkStatusResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from ....commons.errors.organization_not_authorized_error import OrganizationNotAuthorizedError
from ....commons.types.organization_not_authorized_error_message import OrganizationNotAuthorizedErrorMessage
from ....organization_providers.resources.v_2.types.organization_provider_id import OrganizationProviderId
from .errors.expected_network_status_check_error import ExpectedNetworkStatusCheckError
from .types.compute_all_in_network_providers_request import ComputeAllInNetworkProvidersRequest
from .types.compute_all_in_network_providers_response import ComputeAllInNetworkProvidersResponse
from .types.expected_network_status_check_error_message import ExpectedNetworkStatusCheckErrorMessage
from .types.expected_network_status_request_v_2 import ExpectedNetworkStatusRequestV2
from .types.expected_network_status_response_v_2 import ExpectedNetworkStatusResponseV2

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V2Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def compute_for_rendering_provider(
        self, rendering_provider_id: OrganizationProviderId, *, request: ExpectedNetworkStatusRequestV2
    ) -> ExpectedNetworkStatusResponseV2:
        """
        Computes the expected network status for a given rendering provider.
        This endpoint is not available to all customers. Reach out to the Candid sales team
        to discuss enabling this endpoint if it is not available for your organization.

        Parameters:
            - rendering_provider_id: OrganizationProviderId.

            - request: ExpectedNetworkStatusRequestV2.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/expected-network-status/v2/compute/{rendering_provider_id}",
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ExpectedNetworkStatusResponseV2, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "ExpectedNetworkStatusCheckError":
                raise ExpectedNetworkStatusCheckError(
                    pydantic.parse_obj_as(ExpectedNetworkStatusCheckErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "OrganizationNotAuthorizedError":
                raise OrganizationNotAuthorizedError(
                    pydantic.parse_obj_as(OrganizationNotAuthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def compute_all_in_network_providers(
        self, *, request: ComputeAllInNetworkProvidersRequest
    ) -> ComputeAllInNetworkProvidersResponse:
        """
        Computes all the in network providers for a given set of inputs.
        This endpoint is not available to all customers. Reach out to the Candid sales team
        to discuss enabling this endpoint if it is not available for your organization.

        Parameters:
            - request: ComputeAllInNetworkProvidersRequest.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/expected-network-status/v2/compute"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ComputeAllInNetworkProvidersResponse, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "ExpectedNetworkStatusCheckError":
                raise ExpectedNetworkStatusCheckError(
                    pydantic.parse_obj_as(ExpectedNetworkStatusCheckErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "OrganizationNotAuthorizedError":
                raise OrganizationNotAuthorizedError(
                    pydantic.parse_obj_as(OrganizationNotAuthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV2Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def compute_for_rendering_provider(
        self, rendering_provider_id: OrganizationProviderId, *, request: ExpectedNetworkStatusRequestV2
    ) -> ExpectedNetworkStatusResponseV2:
        """
        Computes the expected network status for a given rendering provider.
        This endpoint is not available to all customers. Reach out to the Candid sales team
        to discuss enabling this endpoint if it is not available for your organization.

        Parameters:
            - rendering_provider_id: OrganizationProviderId.

            - request: ExpectedNetworkStatusRequestV2.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/expected-network-status/v2/compute/{rendering_provider_id}",
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ExpectedNetworkStatusResponseV2, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "ExpectedNetworkStatusCheckError":
                raise ExpectedNetworkStatusCheckError(
                    pydantic.parse_obj_as(ExpectedNetworkStatusCheckErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "OrganizationNotAuthorizedError":
                raise OrganizationNotAuthorizedError(
                    pydantic.parse_obj_as(OrganizationNotAuthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def compute_all_in_network_providers(
        self, *, request: ComputeAllInNetworkProvidersRequest
    ) -> ComputeAllInNetworkProvidersResponse:
        """
        Computes all the in network providers for a given set of inputs.
        This endpoint is not available to all customers. Reach out to the Candid sales team
        to discuss enabling this endpoint if it is not available for your organization.

        Parameters:
            - request: ComputeAllInNetworkProvidersRequest.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/expected-network-status/v2/compute"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ComputeAllInNetworkProvidersResponse, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "ExpectedNetworkStatusCheckError":
                raise ExpectedNetworkStatusCheckError(
                    pydantic.parse_obj_as(ExpectedNetworkStatusCheckErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "OrganizationNotAuthorizedError":
                raise OrganizationNotAuthorizedError(
                    pydantic.parse_obj_as(OrganizationNotAuthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .in_network_status import InNetworkStatus
from .indeterminate_network_status import IndeterminateNetworkStatus
from .out_of_network_status import OutOfNetworkStatus


class ExpectedNetworkStatusV2_InNetwork(InNetworkStatus):
    type: typing_extensions.Literal["in_network"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ExpectedNetworkStatusV2_OutOfNetwork(OutOfNetworkStatus):
    type: typing_extensions.Literal["out_of_network"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class ExpectedNetworkStatusV2_Indeterminate(IndeterminateNetworkStatus):
    type: typing_extensions.Literal["indeterminate"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


ExpectedNetworkStatusV2 = typing.Union[
    ExpectedNetworkStatusV2_InNetwork, ExpectedNetworkStatusV2_OutOfNetwork, ExpectedNetworkStatusV2_Indeterminate
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    InsurancePayment,
    InsurancePaymentCreate,
    InsurancePaymentId,
    InsurancePaymentSortField,
    InsurancePaymentsPage,
)

__all__ = [
    "InsurancePayment",
    "InsurancePaymentCreate",
    "InsurancePaymentId",
    "InsurancePaymentSortField",
    "InsurancePaymentsPage",
]



# This file was auto-generated by Fern from our API Definition.

from .contract_is_linked_to_fee_schedule_http_error import ContractIsLinkedToFeeScheduleHttpError

__all__ = ["ContractIsLinkedToFeeScheduleHttpError"]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .....commons.types.npi import Npi
from .....commons.types.tax_id import TaxId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class PayeeIdentifier_Npi(pydantic.BaseModel):
    type: typing_extensions.Literal["npi"]
    value: Npi

    class Config:
        frozen = True
        smart_union = True


class PayeeIdentifier_Tin(pydantic.BaseModel):
    type: typing_extensions.Literal["tin"]
    value: TaxId

    class Config:
        frozen = True
        smart_union = True


class PayeeIdentifier_CmsPlanId(pydantic.BaseModel):
    type: typing_extensions.Literal["cms_plan_id"]
    value: str

    class Config:
        frozen = True
        smart_union = True


PayeeIdentifier = typing.Union[PayeeIdentifier_Npi, PayeeIdentifier_Tin, PayeeIdentifier_CmsPlanId]



# This file was auto-generated by Fern from our API Definition.

from .gender import Gender
from .individual_base import IndividualBase
from .individual_id import IndividualId
from .patient import Patient
from .patient_base import PatientBase
from .patient_create import PatientCreate
from .subscriber import Subscriber
from .subscriber_base import SubscriberBase
from .subscriber_create import SubscriberCreate

__all__ = [
    "Gender",
    "IndividualBase",
    "IndividualId",
    "Patient",
    "PatientBase",
    "PatientCreate",
    "Subscriber",
    "SubscriberBase",
    "SubscriberCreate",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....organization_providers.resources.v_2.types.organization_provider_id import OrganizationProviderId
from .....payers.resources.v_3.types.payer_uuid import PayerUuid
from .explanation import Explanation

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class IndeterminateNetworkStatus(pydantic.BaseModel):
    error: str
    explanation: Explanation
    routed_payer_uuid: typing.Optional[PayerUuid] = None
    routed_billing_provider_id: typing.Optional[OrganizationProviderId] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .....commons.types.regions import Regions

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class RegionsUpdate_Set(pydantic.BaseModel):
    type: typing_extensions.Literal["set"]
    value: Regions

    class Config:
        frozen = True
        smart_union = True


class RegionsUpdate_Remove(pydantic.BaseModel):
    type: typing_extensions.Literal["remove"]

    class Config:
        frozen = True
        smart_union = True


RegionsUpdate = typing.Union[RegionsUpdate_Set, RegionsUpdate_Remove]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.date import Date
from .....identifiers.types.updatable_identifier import UpdatableIdentifier
from ...v_2.types.license_type import LicenseType
from ...v_2.types.organization_provider_address import OrganizationProviderAddress
from ...v_2.types.provider_type import ProviderType

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class OrganizationProviderUpdateV2(pydantic.BaseModel):
    npi: typing.Optional[str] = pydantic.Field(
        default=None,
        description="The NPI of the provider. This must be all digits [0-9] and exactly 10 characters long.",
    )
    is_rendering: typing.Optional[bool] = pydantic.Field(
        default=None, description="Whether the provider can be used to render services."
    )
    is_billing: typing.Optional[bool] = pydantic.Field(
        default=None, description="Whether the provider can be used to bill services."
    )
    first_name: typing.Optional[str] = pydantic.Field(
        default=None, description="The first name of the provider, if the provider is an individual."
    )
    last_name: typing.Optional[str] = pydantic.Field(
        default=None, description="The last name of the provider, if the provider is an individual."
    )
    organization_name: typing.Optional[str] = pydantic.Field(
        default=None, description="The name of the provider, if the provider is an organization."
    )
    provider_type: typing.Optional[ProviderType] = pydantic.Field(
        default=None,
        description="Whether the provider is an individual (NPPES Type 1) or organization (NPPES Type 2) provider.",
    )
    tax_id: typing.Optional[str] = pydantic.Field(
        default=None,
        description="If the provider has a contract with insurance, this must be the same tax ID given to the payer on an IRS W-9 form completed during contracting.",
    )
    taxonomy_code: typing.Optional[str] = pydantic.Field(
        default=None, description="A code designating classification and specialization."
    )
    license_type: typing.Optional[LicenseType] = pydantic.Field(
        default=None, description="The type of license that the provider holds."
    )
    addresses: typing.Optional[typing.List[OrganizationProviderAddress]] = pydantic.Field(
        default=None, description="The addresses associated with this provider."
    )
    employment_start_date: typing.Optional[Date] = pydantic.Field(
        default=None, description="The employment start date for the provider."
    )
    employment_termination_date: typing.Optional[Date] = pydantic.Field(
        default=None, description="The employment termination date for the provider."
    )
    qualifications: typing.Optional[typing.List[UpdatableIdentifier]] = pydantic.Field(
        default=None, description="Provider's qualifications (medicare provider number, medicaid provider number, etc.)"
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .intake_follow_up_id import IntakeFollowUpId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class IntakeFollowUp(pydantic.BaseModel):
    """
    from candid.resources.encounters.v_4 import IntakeFollowUp

    IntakeFollowUp(
        id="4F3D57F9-AC94-49D6-87E4-E804B709917A",
        text="Do you have any allergies?",
        response="No allergies",
    )
    """

    id: IntakeFollowUpId
    text: str
    response: typing.Optional[str] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



import typing

from pydantic.v1 import BaseModel, root_validator

from candid import CandidApiEnvironment
from candid.client import CandidApi, AsyncCandidApi
from candid.resources.auth.resources.v_2 import AuthGetTokenRequest


class CandidApiClientOptions(BaseModel):
    token: typing.Optional[str]
    client_id: typing.Optional[str]
    client_secret: typing.Optional[str]

    @root_validator(pre=False)
    def token_or_client_id_and_secret_present(cls, values):
        if values.get("token") is not None:
            return values
        elif values.get("client_id") is not None and values.get("client_secret") is not None:
            return values
        raise ValueError("Either token or client_id and client_secret must be provided")


def get_token_for_options(options: CandidApiClientOptions, environment: CandidApiEnvironment) -> str:
    if options.token is not None:
        return options.token
    else:
        unauthed_client = CandidApi(environment=environment, token=None)
        response = unauthed_client.auth.v_2.get_token(request=AuthGetTokenRequest(client_id=options.client_id, client_secret=options.client_secret))
        return response.access_token


class CandidApiClient:
    def __init__(
        self, *, options: CandidApiClientOptions, environment: CandidApiEnvironment = CandidApiEnvironment.PRODUCTION
    ):
        candid = CandidApi(token=get_token_for_options(options, environment), environment=environment)
        self.auth = candid.auth
        self.encounters = candid.encounters
        self.billing_notes = candid.billing_notes
        self.expected_network_status = candid.expected_network_status
        self.payers = candid.payers


class AsyncCandidApiClient:
    def __init__(
        self, *, options: CandidApiClientOptions, environment: CandidApiEnvironment = CandidApiEnvironment.PRODUCTION
    ):
        candid = AsyncCandidApi(token=get_token_for_options(options, environment), environment=environment)
        self.auth = candid.auth
        self.encounters = candid.encounters
        self.billing_notes = candid.billing_notes
        self.expected_network_status = candid.expected_network_status
        self.payers = candid.payers



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...insurance_cards.resources.v_2.types.insurance_card import InsuranceCard
from .individual_id import IndividualId
from .subscriber_base import SubscriberBase


class Subscriber(SubscriberBase):
    """
    import uuid

    from candid import (Gender, InsuranceTypeCode,
                        PatientRelationshipToInsuredCodeAll, SourceOfPaymentCode,
                        State, StreetAddressShortZip, Subscriber)
    from candid.resources.insurance_cards.v_2 import InsuranceCard

    Subscriber(individual_id=uuid.UUID("797348a9-e7e8-4e59-8628-95390d079c0b", ), insurance_card=InsuranceCard(insurance_card_id=uuid.UUID("ca5b7711-4419-4161-9b7c-3494ac40c8d4", ), member_id="E85313B4-0FFC-4119-8042-8161A4ECFF0A", payer_name="John Doe", payer_id="836DDAA6-863F-4020-ACCA-205A689F0002", rx_bin="610014", rx_pcn="MEDDPRIME", image_url_front="https://s3.amazonaws.com/front.jpg", image_url_back="https://s3.amazonaws.com/back.jpg", group_number="ABC12345", plan_name="Silver PPO Plan", plan_type=SourceOfPaymentCode.09, insurance_type=InsuranceTypeCode.12, ), patient_relationship_to_subscriber_code=PatientRelationshipToInsuredCodeAll.01, date_of_birth="2000-01-01", address=StreetAddressShortZip(address_1="123 Main St", address_2="Apt 1", city="New York", state=State.NY, zip_code="10001", zip_plus_four_code="1234", ), first_name="John", last_name="Doe", gender=Gender.MALE, )
    """

    individual_id: IndividualId
    insurance_card: InsuranceCard

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .rx_cui import RxCui

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Medication(pydantic.BaseModel):
    """
    from candid.resources.encounters.v_4 import Medication

    Medication(
        name="Lisinopril",
        rx_cui="860975",
        dosage="10mg",
        dosage_form="Tablet",
        frequency="Once Daily",
        as_needed=True,
    )
    """

    name: str
    rx_cui: typing.Optional[RxCui] = None
    dosage: typing.Optional[str] = None
    dosage_form: typing.Optional[str] = None
    frequency: typing.Optional[str] = None
    as_needed: typing.Optional[bool] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from .....core.remove_none_from_dict import remove_none_from_dict
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.http_request_validation_error import HttpRequestValidationError
from ....commons.errors.updates_disabled_due_to_external_system_integration_error import (
    UpdatesDisabledDueToExternalSystemIntegrationError,
)
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.page_token import PageToken
from ....commons.types.request_validation_error import RequestValidationError
from ....commons.types.updates_disabled_due_to_external_system_integration_error_message import (
    UpdatesDisabledDueToExternalSystemIntegrationErrorMessage,
)
from ..v_2.types.organization_provider_id import OrganizationProviderId
from ..v_2.types.organization_provider_sort_options import OrganizationProviderSortOptions
from .types.organization_provider_create_v_2 import OrganizationProviderCreateV2
from .types.organization_provider_page_v_2 import OrganizationProviderPageV2
from .types.organization_provider_update_v_2 import OrganizationProviderUpdateV2
from .types.organization_provider_v_2 import OrganizationProviderV2

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V3Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(self, organization_provider_id: OrganizationProviderId) -> OrganizationProviderV2:
        """
        Parameters:
            - organization_provider_id: OrganizationProviderId.
        ---
        import uuid

        from candid.client import CandidApi

        client = CandidApi(
            token="YOUR_TOKEN",
        )
        client.organization_providers.v_3.get(
            organization_provider_id=uuid.UUID(
                "965a563a-0285-4910-9569-e3739c0f6eab",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/organization-providers/v3/{organization_provider_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationProviderV2, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        search_term: typing.Optional[str] = None,
        npi: typing.Optional[str] = None,
        is_rendering: typing.Optional[bool] = None,
        is_billing: typing.Optional[bool] = None,
        page_token: typing.Optional[PageToken] = None,
        sort: typing.Optional[OrganizationProviderSortOptions] = None,
    ) -> OrganizationProviderPageV2:
        """
        Parameters:
            - limit: typing.Optional[int]. Limit the number of results returned. Defaults to 100.

            - search_term: typing.Optional[str]. Filter to a name or a part of a name.

            - npi: typing.Optional[str]. Filter to a specific NPI.

            - is_rendering: typing.Optional[bool]. Filter to only rendering providers.

            - is_billing: typing.Optional[bool]. Filter to only billing providers.

            - page_token: typing.Optional[PageToken]. The page token to continue paging through a previous request.

            - sort: typing.Optional[OrganizationProviderSortOptions]. Defaults to PROVIDER_NAME_ASC.
        ---
        from candid.client import CandidApi
        from candid.resources.organization_providers.v_2 import (
            OrganizationProviderSortOptions,
        )

        client = CandidApi(
            token="YOUR_TOKEN",
        )
        client.organization_providers.v_3.get_multi(
            limit=100,
            search_term="john",
            npi="1234567890",
            is_rendering=True,
            is_billing=True,
            page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
            sort=OrganizationProviderSortOptions.PROVIDER_NAME_ASC,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/organization-providers/v3"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "search_term": search_term,
                    "npi": npi,
                    "is_rendering": is_rendering,
                    "is_billing": is_billing,
                    "page_token": page_token,
                    "sort": sort,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationProviderPageV2, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(self, *, request: OrganizationProviderCreateV2) -> OrganizationProviderV2:
        """
        Parameters:
            - request: OrganizationProviderCreateV2.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/organization-providers/v3"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationProviderV2, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UpdatesDisabledDueToExternalSystemIntegrationError":
                raise UpdatesDisabledDueToExternalSystemIntegrationError(
                    pydantic.parse_obj_as(UpdatesDisabledDueToExternalSystemIntegrationErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self, organization_provider_id: OrganizationProviderId, *, request: OrganizationProviderUpdateV2
    ) -> OrganizationProviderV2:
        """
        Parameters:
            - organization_provider_id: OrganizationProviderId.

            - request: OrganizationProviderUpdateV2.
        """
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/organization-providers/v3/{organization_provider_id}"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationProviderV2, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UpdatesDisabledDueToExternalSystemIntegrationError":
                raise UpdatesDisabledDueToExternalSystemIntegrationError(
                    pydantic.parse_obj_as(UpdatesDisabledDueToExternalSystemIntegrationErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV3Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(self, organization_provider_id: OrganizationProviderId) -> OrganizationProviderV2:
        """
        Parameters:
            - organization_provider_id: OrganizationProviderId.
        ---
        import uuid

        from candid.client import AsyncCandidApi

        client = AsyncCandidApi(
            token="YOUR_TOKEN",
        )
        await client.organization_providers.v_3.get(
            organization_provider_id=uuid.UUID(
                "965a563a-0285-4910-9569-e3739c0f6eab",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/organization-providers/v3/{organization_provider_id}"
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationProviderV2, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        search_term: typing.Optional[str] = None,
        npi: typing.Optional[str] = None,
        is_rendering: typing.Optional[bool] = None,
        is_billing: typing.Optional[bool] = None,
        page_token: typing.Optional[PageToken] = None,
        sort: typing.Optional[OrganizationProviderSortOptions] = None,
    ) -> OrganizationProviderPageV2:
        """
        Parameters:
            - limit: typing.Optional[int]. Limit the number of results returned. Defaults to 100.

            - search_term: typing.Optional[str]. Filter to a name or a part of a name.

            - npi: typing.Optional[str]. Filter to a specific NPI.

            - is_rendering: typing.Optional[bool]. Filter to only rendering providers.

            - is_billing: typing.Optional[bool]. Filter to only billing providers.

            - page_token: typing.Optional[PageToken]. The page token to continue paging through a previous request.

            - sort: typing.Optional[OrganizationProviderSortOptions]. Defaults to PROVIDER_NAME_ASC.
        ---
        from candid.client import AsyncCandidApi
        from candid.resources.organization_providers.v_2 import (
            OrganizationProviderSortOptions,
        )

        client = AsyncCandidApi(
            token="YOUR_TOKEN",
        )
        await client.organization_providers.v_3.get_multi(
            limit=100,
            search_term="john",
            npi="1234567890",
            is_rendering=True,
            is_billing=True,
            page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
            sort=OrganizationProviderSortOptions.PROVIDER_NAME_ASC,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/organization-providers/v3"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "search_term": search_term,
                    "npi": npi,
                    "is_rendering": is_rendering,
                    "is_billing": is_billing,
                    "page_token": page_token,
                    "sort": sort,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationProviderPageV2, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(self, *, request: OrganizationProviderCreateV2) -> OrganizationProviderV2:
        """
        Parameters:
            - request: OrganizationProviderCreateV2.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/organization-providers/v3"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationProviderV2, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UpdatesDisabledDueToExternalSystemIntegrationError":
                raise UpdatesDisabledDueToExternalSystemIntegrationError(
                    pydantic.parse_obj_as(UpdatesDisabledDueToExternalSystemIntegrationErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self, organization_provider_id: OrganizationProviderId, *, request: OrganizationProviderUpdateV2
    ) -> OrganizationProviderV2:
        """
        Parameters:
            - organization_provider_id: OrganizationProviderId.

            - request: OrganizationProviderUpdateV2.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/organization-providers/v3/{organization_provider_id}"
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationProviderV2, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UpdatesDisabledDueToExternalSystemIntegrationError":
                raise UpdatesDisabledDueToExternalSystemIntegrationError(
                    pydantic.parse_obj_as(UpdatesDisabledDueToExternalSystemIntegrationErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...commons.types.claim_id import ClaimId
from ...era.types.era import Era
from ...service_lines.resources.v_2.types.service_line import ServiceLine
from .claim_status import ClaimStatus

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Claim(pydantic.BaseModel):
    """
    import datetime
    import uuid

    from candid import (Claim, ClaimStatus, DateRangeOptionalEnd, Era,
                        FacilityTypeCode, Invoice, InvoiceItem, InvoiceStatus,
                        ProcedureModifier, ServiceLineUnits)
    from candid.resources.service_lines.v_2 import (DenialReasonContent,
                                                    ServiceLine,
                                                    ServiceLineAdjustment,
                                                    ServiceLineDenialReason,
                                                    ServiceLineEraData)

    Claim(claim_id=uuid.UUID("dd9d7f82-37b5-449d-aa63-26925398335b", ), status=ClaimStatus.BILLER_RECEIVED, clearinghouse="Change Healthcare", clearinghouse_claim_id="5BA7C3AB-2BC2-496C-8B10-6CAC73D0729D", payer_claim_id="9BB9F259-9756-4F16-8F53-9DBB9F7EB1BB", service_lines=[ServiceLine(modifiers=[ProcedureModifier.22], charge_amount_cents=10000, allowed_amount_cents=8000, insurance_balance_cents=0, patient_balance_cents=2000, paid_amount_cents=8000, patient_responsibility_cents=2000, diagnosis_id_zero=uuid.UUID("4ac84bcd-12f5-4f86-a57b-e06749127c98", ), diagnosis_id_one=uuid.UUID("eea5ca5a-8b43-45fd-8cbd-c6cc1103e759", ), diagnosis_id_two=uuid.UUID("5c4aa029-2db9-4202-916e-e93c708f65ff", ), diagnosis_id_three=uuid.UUID("81795126-a3ac-443c-b47e-7259a16ab4a2", ), service_line_era_data=ServiceLineEraData(service_line_adjustments=[ServiceLineAdjustment(created_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), adjustment_group_code="CO", adjustment_reason_code="CO", adjustment_amount_cents=1000, adjustment_note="test_note", )], remittance_advice_remark_codes=["N362"], ), service_line_manual_adjustments=[ServiceLineAdjustment(created_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), adjustment_group_code="CO", adjustment_reason_code="CO", adjustment_amount_cents=1000, adjustment_note="test_note", )], related_invoices=[Invoice(id=uuid.UUID("901be2f1-41bc-456e-9987-4fe2f84f9d75", ), created_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), updated_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), organzation_id=uuid.UUID("f13f73d4-4344-46ea-9d93-33bcffbb9f36", ), source_id="9B626577-8808-4F28-9ED1-F0DFF0D49BBC", source_customer_id="624D1972-8C69-4C2F-AEFA-10856F734DB3", patient_external_id="10FED4D6-4C5A-48DF-838A-EEF45A74788D", note="test_note", due_date="2023-10-10", status=InvoiceStatus.DRAFT, url="https://example.com", customer_invoice_url="https://example.com", items=[InvoiceItem(service_line_id=uuid.UUID("ced00f23-6e68-4678-9dbc-f5aa2969a565", ), amount_cents=500, )], )], denial_reason=ServiceLineDenialReason(reason=DenialReasonContent.AUTHORIZATION_REQUIRED, ), place_of_service_code=FacilityTypeCode.01, service_line_id=uuid.UUID("ced00f23-6e68-4678-9dbc-f5aa2969a565", ), procedure_code="99213", quantity="1", units=ServiceLineUnits.MJ, claim_id=uuid.UUID("026a1fb8-748e-4859-a2d7-3ea9e07d25ae", ), date_of_service_range=DateRangeOptionalEnd(start_date="2023-01-01", end_date="2023-01-03", ), )], eras=[Era(era_id=uuid.UUID("4d844ef1-2253-43cd-a4f1-6db7e65cb54b", ), check_number="CHK12345", check_date="2023-10-12", )], )
    """

    claim_id: ClaimId
    status: ClaimStatus
    clearinghouse: typing.Optional[str] = None
    clearinghouse_claim_id: typing.Optional[str] = None
    payer_claim_id: typing.Optional[str] = None
    service_lines: typing.List[ServiceLine]
    eras: typing.List[Era]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class TaskStatus(str, enum.Enum):
    FINISHED = "finished"
    ADDRESSED_BY_PROVIDER_GROUP = "addressed_by_provider_group"
    SENT_TO_PROVIDER_GROUP = "sent_to_provider_group"
    OPEN_TASK = "open"
    BLOCKED = "blocked"
    WAITING_FOR_REVIEW = "waiting_for_review"
    IN_PROGRESS = "in_progress"

    def visit(
        self,
        finished: typing.Callable[[], T_Result],
        addressed_by_provider_group: typing.Callable[[], T_Result],
        sent_to_provider_group: typing.Callable[[], T_Result],
        open_task: typing.Callable[[], T_Result],
        blocked: typing.Callable[[], T_Result],
        waiting_for_review: typing.Callable[[], T_Result],
        in_progress: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is TaskStatus.FINISHED:
            return finished()
        if self is TaskStatus.ADDRESSED_BY_PROVIDER_GROUP:
            return addressed_by_provider_group()
        if self is TaskStatus.SENT_TO_PROVIDER_GROUP:
            return sent_to_provider_group()
        if self is TaskStatus.OPEN_TASK:
            return open_task()
        if self is TaskStatus.BLOCKED:
            return blocked()
        if self is TaskStatus.WAITING_FOR_REVIEW:
            return waiting_for_review()
        if self is TaskStatus.IN_PROGRESS:
            return in_progress()



# This file was auto-generated by Fern from our API Definition.

from .types import AuthGetTokenRequest, AuthGetTokenResponse, TooManyRequestsErrorType
from .errors import TooManyRequestsError

__all__ = ["AuthGetTokenRequest", "AuthGetTokenResponse", "TooManyRequestsError", "TooManyRequestsErrorType"]



# This file was auto-generated by Fern from our API Definition.

from . import commons, v_3
from .commons import TaskCategory, TaskStatus, TaskType

__all__ = ["TaskCategory", "TaskStatus", "TaskType", "commons", "v_3"]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.datetime_utils import serialize_datetime
from .....core.jsonable_encoder import jsonable_encoder
from .....core.remove_none_from_dict import remove_none_from_dict
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.unauthorized_error import UnauthorizedError
from ....commons.errors.unprocessable_entity_error import UnprocessableEntityError
from ....commons.types.encounter_id import EncounterId
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.page_token import PageToken
from ....commons.types.task_id import TaskId
from ....commons.types.unauthorized_error_message import UnauthorizedErrorMessage
from ....commons.types.unprocessable_entity_error_message import UnprocessableEntityErrorMessage
from ....commons.types.user_id import UserId
from ..commons.types.task_status import TaskStatus
from ..commons.types.task_type import TaskType
from .errors.task_updated_to_deprecated_status_error import TaskUpdatedToDeprecatedStatusError
from .types.task import Task
from .types.task_actions import TaskActions
from .types.task_create_v_3 import TaskCreateV3
from .types.task_page import TaskPage
from .types.task_sort_options import TaskSortOptions
from .types.task_update_v_3 import TaskUpdateV3
from .types.task_updated_to_deprecated_status_error_type import TaskUpdatedToDeprecatedStatusErrorType

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V3Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_actions(self, task_id: TaskId) -> TaskActions:
        """
        Parameters:
            - task_id: TaskId.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/tasks/v3/{task_id}/actions"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TaskActions, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        status: typing.Optional[TaskStatus] = None,
        task_type: typing.Optional[TaskType] = None,
        categories: typing.Optional[str] = None,
        updated_since: typing.Optional[dt.datetime] = None,
        encounter_id: typing.Optional[EncounterId] = None,
        search_term: typing.Optional[str] = None,
        assigned_to_id: typing.Optional[UserId] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        billing_provider_npi: typing.Optional[str] = None,
        sort: typing.Optional[TaskSortOptions] = None,
    ) -> TaskPage:
        """
        Parameters:
            - limit: typing.Optional[int]. Defaults to 100

            - page_token: typing.Optional[PageToken].

            - status: typing.Optional[TaskStatus].

            - task_type: typing.Optional[TaskType].

            - categories: typing.Optional[str]. Only return tasks with categories that match one in this comma-separated list.

            - updated_since: typing.Optional[dt.datetime]. Only return tasks updated on or after this date-time

            - encounter_id: typing.Optional[EncounterId]. Only return tasks associated with this encounter

            - search_term: typing.Optional[str]. Query tasks by encounter_id, claim_id, task_id, or external_id

            - assigned_to_id: typing.Optional[UserId]. Only return tasks assigned to this user

            - date_of_service_min: typing.Optional[dt.date]. The minimum date of service for the linked encounter

            - date_of_service_max: typing.Optional[dt.date]. The maximum date of service for the linked encounter

            - billing_provider_npi: typing.Optional[str]. The NPI of the billing provider associated with the task's claim

            - sort: typing.Optional[TaskSortOptions]. Defaults to updated_at:desc
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/tasks/v3"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "page_token": page_token,
                    "status": status,
                    "task_type": task_type,
                    "categories": categories,
                    "updated_since": serialize_datetime(updated_since) if updated_since is not None else None,
                    "encounter_id": jsonable_encoder(encounter_id),
                    "search_term": search_term,
                    "assigned_to_id": jsonable_encoder(assigned_to_id),
                    "date_of_service_min": str(date_of_service_min) if date_of_service_min is not None else None,
                    "date_of_service_max": str(date_of_service_max) if date_of_service_max is not None else None,
                    "billing_provider_npi": billing_provider_npi,
                    "sort": sort,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TaskPage, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, task_id: TaskId) -> Task:
        """
        Parameters:
            - task_id: TaskId.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/tasks/v3/{task_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Task, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(self, *, request: TaskCreateV3) -> Task:
        """
        Parameters:
            - request: TaskCreateV3.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/tasks/v3"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Task, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(self, task_id: TaskId, *, request: TaskUpdateV3) -> Task:
        """
        Parameters:
            - task_id: TaskId.

            - request: TaskUpdateV3.
        """
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/tasks/v3/{task_id}"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Task, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "TaskUpdatedToDeprecatedStatusError":
                raise TaskUpdatedToDeprecatedStatusError(
                    pydantic.parse_obj_as(TaskUpdatedToDeprecatedStatusErrorType, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV3Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_actions(self, task_id: TaskId) -> TaskActions:
        """
        Parameters:
            - task_id: TaskId.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/tasks/v3/{task_id}/actions"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TaskActions, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        status: typing.Optional[TaskStatus] = None,
        task_type: typing.Optional[TaskType] = None,
        categories: typing.Optional[str] = None,
        updated_since: typing.Optional[dt.datetime] = None,
        encounter_id: typing.Optional[EncounterId] = None,
        search_term: typing.Optional[str] = None,
        assigned_to_id: typing.Optional[UserId] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        billing_provider_npi: typing.Optional[str] = None,
        sort: typing.Optional[TaskSortOptions] = None,
    ) -> TaskPage:
        """
        Parameters:
            - limit: typing.Optional[int]. Defaults to 100

            - page_token: typing.Optional[PageToken].

            - status: typing.Optional[TaskStatus].

            - task_type: typing.Optional[TaskType].

            - categories: typing.Optional[str]. Only return tasks with categories that match one in this comma-separated list.

            - updated_since: typing.Optional[dt.datetime]. Only return tasks updated on or after this date-time

            - encounter_id: typing.Optional[EncounterId]. Only return tasks associated with this encounter

            - search_term: typing.Optional[str]. Query tasks by encounter_id, claim_id, task_id, or external_id

            - assigned_to_id: typing.Optional[UserId]. Only return tasks assigned to this user

            - date_of_service_min: typing.Optional[dt.date]. The minimum date of service for the linked encounter

            - date_of_service_max: typing.Optional[dt.date]. The maximum date of service for the linked encounter

            - billing_provider_npi: typing.Optional[str]. The NPI of the billing provider associated with the task's claim

            - sort: typing.Optional[TaskSortOptions]. Defaults to updated_at:desc
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/tasks/v3"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "page_token": page_token,
                    "status": status,
                    "task_type": task_type,
                    "categories": categories,
                    "updated_since": serialize_datetime(updated_since) if updated_since is not None else None,
                    "encounter_id": jsonable_encoder(encounter_id),
                    "search_term": search_term,
                    "assigned_to_id": jsonable_encoder(assigned_to_id),
                    "date_of_service_min": str(date_of_service_min) if date_of_service_min is not None else None,
                    "date_of_service_max": str(date_of_service_max) if date_of_service_max is not None else None,
                    "billing_provider_npi": billing_provider_npi,
                    "sort": sort,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TaskPage, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    pydantic.parse_obj_as(UnprocessableEntityErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, task_id: TaskId) -> Task:
        """
        Parameters:
            - task_id: TaskId.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/tasks/v3/{task_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Task, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(self, *, request: TaskCreateV3) -> Task:
        """
        Parameters:
            - request: TaskCreateV3.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/tasks/v3"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Task, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(self, task_id: TaskId, *, request: TaskUpdateV3) -> Task:
        """
        Parameters:
            - task_id: TaskId.

            - request: TaskUpdateV3.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/tasks/v3/{task_id}"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Task, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "TaskUpdatedToDeprecatedStatusError":
                raise TaskUpdatedToDeprecatedStatusError(
                    pydantic.parse_obj_as(TaskUpdatedToDeprecatedStatusErrorType, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .payer_info import PayerInfo
from .payer_uuid import PayerUuid

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class PayerIdentifier_PayerInfo(PayerInfo):
    type: typing_extensions.Literal["payer_info"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class PayerIdentifier_PayerUuid(pydantic.BaseModel):
    type: typing_extensions.Literal["payer_uuid"]
    value: PayerUuid

    class Config:
        frozen = True
        smart_union = True


PayerIdentifier = typing.Union[PayerIdentifier_PayerInfo, PayerIdentifier_PayerUuid]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .license_type import LicenseType
from .organization_provider_address import OrganizationProviderAddress
from .provider_type import ProviderType

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class OrganizationProviderBase(pydantic.BaseModel):
    npi: str = pydantic.Field(
        description="The NPI of the provider. This must be all digits [0-9] and exactly 10 characters long."
    )
    is_rendering: bool = pydantic.Field(description="Whether the provider can be used to render services.")
    is_billing: bool = pydantic.Field(description="Whether the provider can be used to bill services.")
    first_name: typing.Optional[str] = pydantic.Field(
        default=None, description="The first name of the provider, if the provider is an individual."
    )
    last_name: typing.Optional[str] = pydantic.Field(
        default=None, description="The last name of the provider, if the provider is an individual."
    )
    organization_name: typing.Optional[str] = pydantic.Field(
        default=None, description="The name of the provider, if the provider is an organization."
    )
    provider_type: ProviderType = pydantic.Field(
        description="Whether the provider is an individual (NPPES Type 1) or organization (NPPES Type 2) provider."
    )
    tax_id: typing.Optional[str] = pydantic.Field(
        default=None,
        description="If the provider has a contract with insurance, this must be the same tax ID given to the payer on an IRS W-9 form completed during contracting.",
    )
    taxonomy_code: typing.Optional[str] = pydantic.Field(
        default=None, description="A code designating classification and specialization."
    )
    license_type: LicenseType = pydantic.Field(description="The type of license that the provider holds.")
    ptan: typing.Optional[str] = pydantic.Field(
        default=None, description="The Provider Transaction Access Number for the provider."
    )
    medicaid_provider_id: typing.Optional[str] = pydantic.Field(
        default=None, description="The medicaid provider ID for the provider."
    )
    addresses: typing.Optional[typing.List[OrganizationProviderAddress]] = pydantic.Field(
        default=None, description="The addresses associated with this provider."
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.decimal import Decimal
from .....commons.types.facility_type_code import FacilityTypeCode
from .....commons.types.procedure_modifier import ProcedureModifier
from .....commons.types.service_line_units import ServiceLineUnits
from .drug_identification import DrugIdentification

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ServiceLineCreate(pydantic.BaseModel):
    modifiers: typing.Optional[typing.List[ProcedureModifier]] = None
    procedure_code: str
    quantity: Decimal = pydantic.Field(
        description=(
            "String representation of a Decimal that can be parsed by most libraries.\n"
            "A ServiceLine quantity cannot contain more than one digit of precision.\n"
            "Example: 1.1 is valid, 1.11 is not.\n"
        )
    )
    units: ServiceLineUnits
    charge_amount_cents: typing.Optional[int] = pydantic.Field(
        default=None,
        description=(
            "The total amount charged for this service line taking quantity into account. For example, if a single unit\n"
            "costs 100 cents and 2 units were rendered, the charge_amount_cents should be 200. Should be greater than or\n"
            "equal to 0.\n"
        ),
    )
    diagnosis_pointers: typing.List[int] = pydantic.Field(
        description="Indices (zero-indexed) of all the diagnoses this service line references"
    )
    drug_identification: typing.Optional[DrugIdentification] = None
    place_of_service_code: typing.Optional[FacilityTypeCode] = None
    description: typing.Optional[str] = pydantic.Field(
        default=None,
        description="A free-form description to clarify the related data elements and their content. Maps to SV1-01, C003-07 on the 837-P.",
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}
